/* Sistemas Integrados TECNISA
   Compras e Suprimentos

Nome: CUSTO_DO_PRATO.SQL

Descrio:
Retorna o Custo do Prato.

P_CDFILIAL     - Cdigo da Filial (utilizado para pesquisar os
                 parmetros de utilizao dos fatores de correo e
                 coco alem do custo dos produtos.
P_CDFILICUSTO  - Cdigo da Filial (utilizado na funo de custo dos
                 produtos.
P_NRTIPOCUSTO  - Nmero do Tipo de Custo.
P_DTREFERENCIA - Fator de Coco.
P_CDFILAUXPRAT - Cdigo da filial do prato.
P_CDPRATO      - Cdigo do Prato.
P_CDSERVFILI   - Cdigo do Servico.

Criao:
20/01/99 - Ederson

Manutenes:
__/__/99 - _______ - ___________

*/
IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'CUSTO_DO_PRATO_PLA'  AND
	        TYPE = 'P' 			        AND
	        U.UID = O.UID             		        AND
	        U.NAME = USER)

  DROP PROCEDURE CUSTO_DO_PRATO_PLA
  GO

CREATE PROCEDURE CUSTO_DO_PRATO_PLA @P_CDFILIAL     VARCHAR(4),
                                    @P_CDFILICUSTO  VARCHAR(4),
                                    @P_NRTIPOCUSTO  NUMERIC(10),
                                    @P_DTREFERENCIA DATETIME,
                                    @P_CDFILAUXPRAT VARCHAR(4),
                                    @P_CDPRATO      VARCHAR(10),
                                    @P_CDSERVFILI   VARCHAR(3),
                                    @P_CUSTO_PRATO NUMERIC(15,6) OUTPUT


AS

   DECLARE @V_IDUTILFATCOR   VARCHAR(1);
   DECLARE @V_IDUTILFATCOC   VARCHAR(1);
   DECLARE @V_IDTIPCALCFAT   VARCHAR(1);

   DECLARE @V_CDPRODUTO   VARCHAR(10);
   DECLARE @V_QTPRODUTO   NUMERIC(15,6);
   DECLARE @V_CUSTO_PRATO NUMERIC(15,6);
   DECLARE @V_QTPRODREC   NUMERIC(15,6);
   DECLARE @V_SGUNIDADE   VARCHAR(5);
   DECLARE @V_VRPEFATCOC  NUMERIC(15,6);
   DECLARE @V_VRPEFATCOR  NUMERIC(15,6);

   DECLARE @V_CUSTO_DO_PRODUTO NUMERIC(15,6);
   DECLARE @V_QTDE_UNIDADE_ESTOQUE NUMERIC(15,6);

BEGIN

   /***QTDE_BRUTA_PRODUTO(P_CDFILIAL, QTDE_UNIDADE_ESTOQUE(QTPRODRECSER, SGUNIDADE, CDPRODUTO), VRPEFATCOCSE, VRPEFATCORSE, V_IDUTILFATCOR, V_IDUTILFATCOC, V_IDTIPCALCFAT) AS QTPRODUTO***/
   DECLARE C_CUSTO_DO_PRATO CURSOR FOR
   (SELECT
       CDPRODUTO,
       QTBRUTASE AS QTPRODUTO
    FROM
       RECESERV
    WHERE
       (CDFILAUXPRAT = @P_CDFILAUXPRAT) AND
       (CDPRATO      = @P_CDPRATO)      AND
       (CDFILIAL     = @P_CDFILIAL)     AND
       (CDSERVFILI   = @P_CDSERVFILI)

    UNION

    SELECT
       CDPRODUTO,
       QTBRUTALO AS QTPRODUTO
    FROM
       RECEFILI
    WHERE
       (CDFILAUXPRAT = @P_CDFILAUXPRAT) AND
       (CDPRATO      = @P_CDPRATO)      AND
       (CDFILIAL     = @P_CDFILIAL)     AND
        NOT EXISTS (SELECT
                       CDPRATO
                    FROM
                       RECESERV
                    WHERE
                       (CDFILAUXPRAT = @P_CDFILAUXPRAT) AND
                       (CDPRATO      = @P_CDPRATO)      AND
                       (CDFILIAL     = @P_CDFILIAL)     AND
                       (CDSERVFILI   = @P_CDSERVFILI)
                    )
    UNION

    SELECT
       CDPRODUTO,
       QTBRUTAPR  AS QTPRODUTO
    FROM
       RECEPADR
    WHERE
       (CDFILAUXPRAT = @P_CDFILAUXPRAT) AND
       (CDPRATO      = @P_CDPRATO)      AND
        NOT EXISTS (SELECT
                       CDPRATO
                    FROM
                       RECEFILI
                    WHERE
                       (CDFILAUXPRAT = @P_CDFILAUXPRAT) AND
                       (CDPRATO      = @P_CDPRATO)      AND
                       (CDFILIAL     = @P_CDFILIAL)
                    ) AND
        NOT EXISTS (SELECT
                       CDPRATO
                    FROM
                       RECESERV
                    WHERE
                       (CDFILAUXPRAT = @P_CDFILAUXPRAT) AND
                       (CDPRATO      = @P_CDPRATO)      AND
                       (CDFILIAL     = @P_CDFILIAL)     AND
                       (CDSERVFILI   = @P_CDSERVFILI)
                    )
   );


   SELECT
      @V_IDUTILFATCOR = MAX(IDUTILFATCOR),
      @V_IDUTILFATCOC = MAX(IDUTILFATCOC),
      @V_IDTIPCALCFAT = MAX(IDTIPCALCFAT)
   FROM
      PARAMPLANEJA
   WHERE
      CDFILIAL = @P_CDFILIAL;

   IF @V_IDUTILFATCOR IS NULL
   BEGIN
      SET @V_IDUTILFATCOC = 'S';
      SET @V_IDUTILFATCOR = 'S';
      SET @V_IDTIPCALCFAT = 'T';
   END;

   SET @V_CUSTO_PRATO = 0;

   OPEN C_CUSTO_DO_PRATO;

   FETCH NEXT FROM C_CUSTO_DO_PRATO INTO @V_CDPRODUTO, @V_QTPRODUTO;

   WHILE (@@FETCH_STATUS = 0)
   BEGIN

      EXEC CUSTO_DO_PRODUTO_PLA @P_CDFILICUSTO, @V_CDPRODUTO, @P_NRTIPOCUSTO, @P_DTREFERENCIA,
           @P_CUSTO = @V_CUSTO_DO_PRODUTO OUTPUT;

      SET @V_CUSTO_PRATO = @V_CUSTO_PRATO + (ROUND(@V_QTPRODUTO, 5) * ROUND(@V_CUSTO_DO_PRODUTO, 5));

      FETCH NEXT FROM C_CUSTO_DO_PRATO INTO @V_CDPRODUTO, @V_QTPRODUTO;
   END;

   CLOSE C_CUSTO_DO_PRATO;
   DEALLOCATE C_CUSTO_DO_PRATO;

   /*RETURN(ROUND(V_CUSTO_PRATO, 5));*/
   SET @P_CUSTO_PRATO = ROUND(@V_CUSTO_PRATO, 5);
END;
GO

