(function () {

	var ZeedhiDirectives = angular.module("ZeedhiDirectives");

	ZeedhiDirectives.directive('zhVirtualScrollList', function (templateManager) {
		return {
			link: function ($scope, element, attrs, controller) {
				var _updateTemplateAfterScrollCalc = Util.buildDebounceMethod(function _updateTemplateAfterScrollCalc() {
					templateManager.updateTemplate();
				}, 20);

				$scope.widget._getVisibilityParams = function () {
					var listContainerHeight = element.outerHeight();
					var rowHeight = $scope.widget.rowHeight;

					$scope.widget._visibilityParams = {
						rowHeight: rowHeight,
						maxVisibleRows: Math.floor(listContainerHeight / rowHeight)
					};
					increaseMaxVisibleRowsUntilThereIsNoBottomBlankSpace(listContainerHeight, rowHeight);
					adjustVisibiliParamsToGroupedDataLimit($scope.widget._visibilityParams);
				};

				function getVisibleFieldsCount() {
					return $scope.widget.fields.filter(function (field) {
						return field.isVisible && field.aclIsVisible;
					}).length;
				}

				function increaseMaxVisibleRowsUntilThereIsNoBottomBlankSpace(listContainerHeight, rowHeight) {
					var visibilityParams = $scope.widget._visibilityParams;
					while (rowHeight * visibilityParams.maxVisibleRows < listContainerHeight) {
						visibilityParams.maxVisibleRows++;
					}
				}

				function adjustVisibiliParamsToGroupedDataLimit(visibilityParams) {
					var topRow = $scope.widget._topRow || 0;
					var groupedData = $scope.widget.groupedData || [];
					var groupedDataSlice = groupedData.slice(topRow, topRow + visibilityParams.maxVisibleRows);
					visibilityParams.maxVisibleRows = Math.min(visibilityParams.maxVisibleRows, groupedDataSlice.length);
				}

				$scope.widget._calcScrollAreas = function _calcScrollAreas() {
					var totalHeight = 0;
					var listContainer = element;
					var visibilityParams = $scope.widget._visibilityParams;

					if ($scope.widget._hiddenRows > 0) {
						totalHeight = ($scope.widget._hiddenRows * visibilityParams.rowHeight);
					}

					var top = element.find('.list-top-scroll-area');
					var bottom = element.find('.list-bottom-scroll-area');

					if ((top.height() !== listContainer.scrollTop()) || (top.height() + bottom.height() !== totalHeight)) {
						top.height(Math.min(listContainer.scrollTop(), totalHeight));
						bottom.height(totalHeight - top.height());
					}
				};

				$scope.widget._calcTopRow = function _calcTopRow() {
					var listContainer = element;
					$scope.widget._topRow = Math.round(listContainer.scrollTop() / $scope.widget._visibilityParams.rowHeight);
					$scope.widget._updateVisibleRows();
					if (!Util.isMobile()) {
						$scope.widget._calcScrollAreas();
					}

					_updateTemplateAfterScrollCalc();
				};

				var _debounceCalcTopRow = Util.buildDebounceMethod($scope.widget._calcTopRow, 200);
				var _throttleCalcTopRow = Util.buildThrottleMethod($scope.widget._calcTopRow, 110, $scope.widget);
				
				if (!Util.isMobile()) {
					element.on('scroll', _throttleCalcTopRow);
				} else {
					element.on('scroll', _debounceCalcTopRow);
				}
			}
		};
	});

})();